/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2013, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"net_reg_.h"

void
net_reg (struct IN In, struct TESS *pTess, struct REG* pReg)
{
  int DelId = 0;
  double scale = 1;
  double** bbox = NULL;

  if ((*pTess).Dim == 1)
    return;

  if (In.dim == 2)
  {
    // In case of a 2D tessellation, *pTess is actually 3D columar with
    // a very small size in z, which is harmful for regularization.
    // Giving it a large size for regularization and resetting size
    // afterwards.

    bbox = ut_alloc_2d (3, 2);
    neut_tess_bbox (*pTess, bbox);

    scale = ut_num_min (bbox[0][1] - bbox[0][0],
	                bbox[1][1] - bbox[1][0])
	    / (bbox[2][1] - bbox[2][0]);

    ut_free_2d (bbox, 3);

    neut_tess_scale (pTess, 1, 1, scale);
  }

  net_reg_init (In, *pTess, pReg);

  ut_print_message (0, 2, "");

  /*********************************************************************** 
   * if fmax > 0 & mloop >= 1, small edge deletion */

  if ((*pReg).fmax > 1e-15 && (*pReg).mloop >= 1)
    net_reg_merge (pTess, *pReg, &DelId);

  /*---------------------------------------------------------------------- 
   * compressing */
  neut_tess_compress (pTess);

  neut_tess_init_domtessface (pTess);
  neut_tess_init_domtessedge (pTess);
  neut_tess_init_domtessver (pTess);

  /*********************************************************************** 
   * testing full tessellation */

#ifdef HAVE_DEBUGGING
  if (neut_tess_test (*pTess, 0) != 0 || neut_tess_test_dom (*pTess, 0) != 0)
    ut_print_message (2, 2, "The tessellation is not valid.\n");
#endif

  printf ("\n");

  if (In.dim == 2)
    neut_tess_scale (pTess, 1, 1, 1. / scale);

  return;
}
