/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2013, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "net_tess_.h"

int
net_tess_standard (struct TESS PTess, int cell, struct GERMSET
		   *pGermSet, int TessId, struct TESS *pTess)
{
  if (PTess.Dim >= 2)
    net_tess3d (PTess, cell, pGermSet, TessId, pTess, (*pGermSet).N >= 3000);

  else if (PTess.Dim == 1)
    net_tess1d (PTess, cell, *pGermSet, TessId, pTess);

  return 0;
}

int
net_tess_centroid (struct IN In, struct TESS PTess, int cell,
		   struct GERMSET *pGermSet, int TessId,
		   struct TESS *pTess)
{
  int status, iter;
  double rdistmax = -1;
  char *prevmessage = ut_alloc_1d_char (1000);
  char *message = ut_alloc_1d_char (1000);
  double** coo = ut_alloc_2d ((*pGermSet).N + 1, 3);

  status = -1;
  for (iter = 1; status != 0 && iter <= In.centroiditermax; iter++)
  {
    // updating germ coos
    if (iter > 1)
      net_tess_centroid_germupdate (*pTess, coo, In.centroidfact, pGermSet);

    if (PTess.Dim >= 2)
      net_tess3d (PTess, cell, pGermSet, TessId, pTess, 0);
    else if (PTess.Dim == 1)
      net_tess1d (PTess, cell, *pGermSet, TessId, pTess);

    net_tess_centroid_coo (*pTess, coo);

    status = net_tess_centroid_test ((*pGermSet).Dim, *pTess, coo, In.centroidconv, &rdistmax);

    sprintf (message, "iter=%d, dist=%.3g", iter, rdistmax);
    ut_print_progress (stdout, 0, 1, message, prevmessage);
  }

  ut_free_2d (coo, (*pGermSet).N + 1);
  ut_free_1d_char (prevmessage);
  ut_free_1d_char (message);

  return 0;
}
