/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2013, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"net_polycomp_.h"
#include<ANN/ANN.h>

/* PolyhedronCalculation searches out the polyhedron associated
* to the considered germ.
*/
void
PolyhedronCalculation (struct POLY Domain, struct GERMSET *pGermSet,
		       ANNkd_tree* kdTree, int PolyId, struct POLY
		       *pPoly,
		       int* pneighqty)
{
  int i;
  int *NeiId = ut_alloc_1d_int ((*pGermSet).N + 1);
  struct POLYMOD Polymod;
  double* dist = ut_alloc_1d ((*pGermSet).N + 1);
  int NeiQty;
  double wmax = 0;

  // in input, *pneighqty is taken as the average of the number of
  // neighbours used in previous computations. If 0, a default value of
  // 100 is taken instead. In output, the number of neighbours used is
  // returned.

  if (*pneighqty <= 0)
    NeiQty = 100;
  else
    NeiQty = 2 * (*pneighqty);

  wmax = ut_array_1d_max ((*pGermSet).GermWeight + 1, (*pGermSet).N);

  neut_polymod_set_zero (&Polymod);

  neut_poly_polymod (Domain, &Polymod);

  // Standard Voronoi tessellation
  if (!(*pGermSet).LamEq)
  {
    kdTree->annkSearch ((*pGermSet).GermCoo[PolyId],
	                ut_num_min_int ((*pGermSet).N, NeiQty),
			NeiId + 1, dist + 1, 0);

    ut_array_1d_int_addval (NeiId + 1, (*pGermSet).N, 1, NeiId + 1);

    for (i = 2; i <= (*pGermSet).N; i++)
    {
      // The number of recorded neighbours is exceeded.  Recomputing a
      // larger number of neighbours.
      if (i > NeiQty)
      {
	NeiQty *= 2;
	kdTree->annkSearch ((*pGermSet).GermCoo[PolyId],
			    ut_num_min_int ((*pGermSet).N, NeiQty),
			    NeiId + 1, dist + 1, 0);
	ut_array_1d_int_addval (NeiId + 1, (*pGermSet).N, 1, NeiId + 1);
      }

      // CurrentPolyTest checks if the germ is close enough to be able to
      // modify Polymod
      if (CurrentPolyTest (Polymod,
			   (*pGermSet).GermCoo[PolyId],
			   (*pGermSet).GermWeight[PolyId],
			   (*pGermSet).GermCoo[NeiId[i]],
			   (*pGermSet).GermWeight[NeiId[i]]) == 1)
      {
	// NewPolyhedron can modify Polymod using the new germ
	NewPolyhedron ((*pGermSet), PolyId, NeiId[i], &Polymod);
      }
      
      else
      {
	// Testing is given wmax, Polymod still can be modified
	if (CurrentPolyTest (Polymod,
			     (*pGermSet).GermCoo[PolyId],
			     (*pGermSet).GermWeight[PolyId],
			     (*pGermSet).GermCoo[NeiId[i]],
			     wmax) == 0)
	  break;
      }
    }

    (*pneighqty) = i - 1;

    net_polymod_poly (Polymod, pPoly);
  }

  // Lamellae tessellation
  else
  {
    // Initializing cut plane
    double *eq = ut_alloc_1d (4);

    ut_array_1d_memcpy (eq, 4, (*pGermSet).LamEq);

    if (PolyId > 1)
    {
      eq[0] = (*pGermSet).LamEq[0] + (PolyId - 2) * (*pGermSet).LamWidth;
      NewPolyhedron_plane (PolyId - 1, 1, eq, &Polymod);
    }

    // up plane
    if (PolyId < (*pGermSet).N)
    {
      eq[0] = (*pGermSet).LamEq[0] + (PolyId - 1) * (*pGermSet).LamWidth;
      NewPolyhedron_plane (PolyId + 1, -1, eq, &Polymod);
    }

    net_polymod_poly (Polymod, pPoly);
    net_poly_centroid (*pPoly, (*pGermSet).GermCoo[PolyId]);

    ut_free_1d (eq);
  }

  neut_polymod_free (&Polymod);

  ut_free_1d_int (NeiId);
  ut_free_1d (dist);

  return;
}
