/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2013, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "neut_germset_.h"

void
neut_germset_set_zero (struct GERMSET *pGermSet)
{
  (*pGermSet).N = 0;
  (*pGermSet).N1d = 0;
  (*pGermSet).Id = 0;
  (*pGermSet).Dim = 3;
  (*pGermSet).weight = NULL;
  (*pGermSet).morpho = NULL;
  (*pGermSet).Random = 0;
  (*pGermSet).GermCoo = NULL;
  (*pGermSet).GermWeight = NULL;
  (*pGermSet).NDensity = 0;
  (*pGermSet).LamEq = NULL;
  (*pGermSet).LamWidth = 0;

  (*pGermSet).rngid = NULL;
  (*pGermSet).q = NULL;

  // Domain neighbourhood info
  (*pGermSet).Size = NULL;
  (*pGermSet).nN = 0;
  (*pGermSet).nSize = NULL;
  (*pGermSet).nVol = 0;
  (*pGermSet).nGermCoo = NULL;
  (*pGermSet).nGermRandId = NULL;
  (*pGermSet).nMaster = NULL;

  return;
}

void
neut_germset_free (struct GERMSET *pGermSet)
{
  ut_free_1d_char ((*pGermSet).weight);
  ut_free_1d_char ((*pGermSet).morpho);
  ut_free_2d ((*pGermSet).GermCoo, (*pGermSet).N + 1);
  ut_free_1d ((*pGermSet).GermWeight);
  ut_free_1d ((*pGermSet).LamEq);

  ut_free_1d_int ((*pGermSet).rngid);
  ut_free_2d ((*pGermSet).q, (*pGermSet).N + 1);

  // Domain neighbourhood info
  ut_free_2d ((*pGermSet).Size, 3);
  ut_free_2d ((*pGermSet).nSize, 3);
  ut_free_2d ((*pGermSet).nGermCoo, (*pGermSet).nN);
  ut_free_1d_int ((*pGermSet).nGermRandId);
  ut_free_1d_int ((*pGermSet).nMaster);

  neut_germset_set_zero (pGermSet);

  return;
}

int
neut_germset_germ_master (struct GERMSET GSet, int germ)
{
  if (germ <= GSet.N)
    return germ;
  else
    return GSet.nMaster[germ - GSet.N];
}
