/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2013, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#ifndef STRUCTGERMSET_H
#define STRUCTGERMSET_H

struct GERMSET
{
  /// \brief Identifier of the distribution
  int Id;

  /// \brief Dimensionality
  int Dim;

  /// \brief Number of germs.
  int N;

  /// \brief size of the germ set (= bounding box of the domain)
  double **Size;

  /// \brief weight
  /// \details can be: custom or a statistics law or @file_name
  char *weight;

  /// \brief morphology
  /// \details can be: poisson, cube, dodeca, tocta or custom
  char *morpho;

  /// \brief Number used for the germ generation process.
  long Random;
  long RandomO;

  /// \brief Equation for lamella tessellations
  /// \details (equation of the first lamella interface)
  ///          LamEq[i][1]*X1+LamEq[i][2]*X2+LamEq[i][3]*X3=LamEq[i][0]
  double *LamEq;

  /// \brief Width for lamella tessellations
  double LamWidth;

  // GERM DISTRIBUTION -------------------------------------------------

  /// \brief Variant of N which applies to cube and tocta tessellations only
  /// \details number of germs in 1D
  int N1d;

  /// \brief Variant of N which applies to subdomain tessellations only
  /// \details nominal density
  int NDensity;

  // For germ i (i=1...N):
  /// \brief Coordinates of the germs
  /// \details the first index is the germ id (ranging 1-N)
  ///          and the second is the dimension (ranging 0-2)
  double **GermCoo;

  double *GermWeight;

  int *rngid;

  double **q;

  // DOMAIN NEIGHBOURHOOD GERM DISTRIBUTION ----------------------------

  /// Number of germs in the domain neighbourhood
  int nN;

  /// Size of the domain neighbourhood
  double **nSize;

  /// Volume
  double nVol;

  /// nGermCoo[i][j] (j=0...2) are the 3 coordinates.
  double **nGermCoo;

  /// \brief Master of the germ (it is inside the domain)
  /// \details This variable is specific to periodic tessellations
  /// For germ i (i=1...nN):
  int *nMaster;

  // \brief Germs rng output id
  // \details this variable is specific to sudomain-type tessellations
  // For germ i (i=1...nN):
  int *nGermRandId;
};
typedef struct GERMSET GERMSET;

#endif /* STRUCTGERMSET_H */
